﻿unit dataset;

interface
uses
  SysUtils, Classes, setting, Vcl.Dialogs;

  //Str から　ChkStr1～ChkStr2　の間にある文字列を取得する
  function Between(ChkStr1, ChkStr2, Str:string):string;

  //Amazon　Webサービスよりデータ取得
  function WebServiceDataSet(FileName : string) : Boolean;

implementation

//Str から　ChkStr1～ChkStr2　の間にある文字列を取得する
function Between(ChkStr1, ChkStr2, Str:string):string;
var
  WKData : string;
begin
  WKData := Str;
  if (0 <> Pos(ChkStr1, Str)) then
  begin
    WKData := Copy(WKData, Pos(ChkStr1, WKData) + Length(ChkStr1),
              Length(WKData) - Pos(ChkStr1, WKData) + Length(ChkStr1));
  end;
  if 0 <> Pos(ChkStr2, WKData) then
    WKData := Copy(WKData, 1, Pos(ChkStr2, WKData) - 1);
  Result := WKData;
end;

{=== ダウンロードしたHTMLよりデータ取得 =============================}
function WebServiceDataSet(FileName : string) : Boolean;
var
  List1 : TStringList;
  S, Day, Y, M, D: string;
begin
  List1 := TStringList.Create;

  try

    result := False;

    //ファイル有無
    if FileExists(FileName) then List1.LoadFromFile(FileName,TEncoding.UTF8) else exit;
    if List1.Text = '' then exit;


    //詳細リンク
    if (0 <> Pos('"DetailPageURL":"', List1.Text)) then
    begin
      Data.Link := Between('"DetailPageURL":"', '"', List1.Text);
    end;

    //画像
    if (0 <> Pos('"Height":', List1.Text)) then
    begin
      S := Between('"Height":', ',', List1.Text);
      S := '"Height":' + S + ',"URL":"';
      Data.FrontCover := Between(S, '"', List1.Text);
    end;

    //書籍用　作者
    if (0 <> Pos('"Name":"', List1.Text)) then
    begin
      Data.Author1 := Between('"Name":"', '"', List1.Text);
    end;

    //発行所
    if (0 <> Pos('"Manufacturer":{"DisplayValue":"', List1.Text)) then
    begin
      Data.Publication := Between('"Manufacturer":{"DisplayValue":"', '"', List1.Text);
    end;

    //サイズ（メディア形式）
    if (0 <> Pos('"Binding":{"DisplayValue":"', List1.Text)) then
    begin
      Data.Size := Between('"Binding":{"DisplayValue":"', '"', List1.Text);
    end;

    //ページ数
    if (0 <> Pos('"PagesCount":{"DisplayValue":', List1.Text)) then
    begin
      Data.Page := Between('"PagesCount":{"DisplayValue":', ',', List1.Text);
    end;

    //初版発行日
    Day := '';

    if (Day = '') then
    begin
      if (0 <> Pos('"PublicationDate":{"DisplayValue":"', List1.Text)) then
      begin
        Day := Between('"PublicationDate":{"DisplayValue":"', 'T', List1.Text);
      end;
    end;

    if (Day = '') then
    begin
      if (0 <> Pos('"ReleaseDate":{"DisplayValue":"', List1.Text)) then
      begin
        Day := Between('"ReleaseDate":{"DisplayValue":"', 'T', List1.Text);
      end;
    end;

    if Day <> '' then
    begin
      Y := Copy(Day, 1, 4); //年
      if 0 <> Pos(Y + '-', Day) then M := Copy(Day, 6, 2); //月
      if 0 <> Pos(Y + '-' + M + '-', Day) then D := Copy(Day, 9, 2); //日
      Data.FirstY := Y;
      Data.FirstM := M;
      Data.FirstD := D;
      Data.ScheduleY := Y;
      Data.ScheduleM := M;
      Data.ScheduleD := D;
    end;

    //タイトル
    if (0 <> Pos('"Title":{"DisplayValue":"', List1.Text)) then
    begin
      Data.Title := BetWeen('"Title":{"DisplayValue":"', '"', List1.Text);
    end;

    //価格
    if (0 <> Pos('"Price":{"Amount":', List1.Text)) then
    begin
      Data.Price := Between('"Price":{"Amount":', '.', List1.Text);
    end;

    Result := True;

  finally
    List1.Free;

  end;
end;

{=== ダウンロードしたHTMLよりデータ取得 =============================}
function HTMLDataSet_Image(FileName : string) : Boolean;
var
  List1 : TStringList;
  WkData01: string;

begin
  result := False;
  List1 := TStringList.Create;
  try
    //ファイル有無
    if FileExists(FileName) then List1.LoadFromFile(FileName) else exit;
    if List1.Text = '' then exit;

    if (0 <> Pos('<div id="imageViewerDiv">', List1.Text)) then
    begin
      WkData01 := Between('<div id="imageViewerDiv">', '</div>', List1.Text);
      WkData01 := Between('<img src="', '"', WkData01);
    end;

    if (0 <> Pos('<div style="text-align: center;">', List1.Text)) then
    begin
      WkData01 := Between('<div style="text-align: center;">', '</div>', List1.Text);
      WkData01 := Between('<img src="', '"', WkData01);
    end;

    //画像がありません表示の場合
    if (0 <> Pos('.gif', WkData01)) then exit;

    Data.FrontCover := WkData01;

    result := True;

  finally
    List1.Free;
  end;
end;


end.
