﻿unit paapiv5assist;
interface

uses
  Windows, SysUtils, ShellApi,Vcl.Forms,Vcl.Dialogs,Classes,DateUtils, StrUtils,
  Net.HttpClient, Net.URLClient, IOUtils,
  System.JSON, setting, System.Hash;

function GetItemamazon(ASIN:String; SFile:String):Boolean;

function BytesToHex(const bytes: TBytes): string;
function CalcSHA256(const data: string): string;
function CalcHMACSHA256(const data: string; key: TBytes): TBytes;

implementation

//amazonの証明書を作ってデータをダウンロードまでする
function GetItemamazon(ASIN:String; SFile:String):Boolean;
var
  i : integer;
  //LhInstance : Cardinal;
  //GoCURL : String;

  AK,SK,host,path,HMACAlgorithm,regionName,serviceName,aws4Request,payload,signedHeaders : String;

  ParamK   : array [1..5] of String;
  ParamV   : array [1..5] of String;

  kSecret,kDate, kRegion, kService, kSigning, ksignature : TBytes;

  stringToSign, canonicalURL,strHexSignature : String;

  datetime:TDatetime;
  sDate,cDate:String;

  JSON: TJSONObject;
  JSONArray1: TJSONArray;
  JSONArray2: TJSONArray;

  HttpClient: THttpClient;
  Response: IHTTPResponse;
  Header: TNetHeader;
  Headers: TNetHeaders;
  AuthString: string;
  Stream: TStream;

begin

  AK := ''; //Access Key AWSより取得
  SK := ''; //Secret Key AWSより取得
  host := 'webservices.amazon.co.jp';
  path := '/paapi5/getitems';
  HMACAlgorithm := 'AWS4-HMAC-SHA256';
  regionName := 'us-west-2';
  serviceName := 'ProductAdvertisingAPI';
  aws4Request := 'aws4_request';


  //JSONcbjectとJSONArrayを使う
  JSON := TJSONObject.Create;
  JSONArray1 := TJSONArray.Create;
  JSONArray2 := TJSONArray.Create;

  try
    JSONArray1.Add(ASIN);
    JSON.AddPair('ItemIds', TJSONNumber.Create(JSONArray1.ToString));

    JSONArray2.Add('Images.Primary.Small');
    JSONArray2.Add('Images.Primary.Medium');
    JSONArray2.Add('Images.Primary.Large');
    JSONArray2.Add('ItemInfo.ByLineInfo');
    JSONArray2.Add('ItemInfo.ContentInfo');
    JSONArray2.Add('ItemInfo.ContentRating');
    JSONArray2.Add('ItemInfo.Classifications');
    JSONArray2.Add('ItemInfo.Features');
    JSONArray2.Add('ItemInfo.ManufactureInfo');
    JSONArray2.Add('ItemInfo.ProductInfo');
    JSONArray2.Add('ItemInfo.Title');
    JSONArray2.Add('ItemInfo.TradeInInfo');
    JSONArray2.Add('Offers.Listings.Price');

    JSON.AddPair('Resources', TJSONNumber.Create(JSONArray2.ToString));
    JSON.AddPair('PartnerTag', TJSONNumber.Create('"ekakinscribbl-22"'));
    JSON.AddPair('PartnerType', TJSONNumber.Create('"Associates"'));
    JSON.AddPair('Marketplace', TJSONNumber.Create('"www.amazon.co.jp"'));
    JSON.AddPair('Operation', TJSONNumber.Create('"GetItems"'));
    payload := JSON.ToString;

  finally
    JSON.Free;
    JSONArray1.Free;
    JSONArray2.Free;
  end;

  //日時の取得
  datetime := TTimeZone.Local.ToUniversalTime(now);
  sDate := String(FormatDateTime('YYYYMMDD"T"HHNNSS"Z"',datetime));
  cDate := String(FormatDateTime('YYYYMMDD', datetime));
  canonicalURL := 'POST' + #10;
  canonicalURL := canonicalURL + path + #10 + #10;

  ParamK[1] := 'content-encoding';
  ParamV[1] := 'amz-1.0';

  ParamK[2] := 'content-type';
  ParamV[2] := 'application/json; charset=utf-8';

  ParamK[3] := 'host';
  ParamV[3] := host;

  ParamK[4] := 'x-amz-date';
  ParamV[4] := sDate;

  ParamK[5] := 'x-amz-target';
  ParamV[5] := 'com.amazon.paapi5.v1.ProductAdvertisingAPIv1.GetItems';

  signedHeaders := '';
  for i := 1 to 5 do
  begin
    signedHeaders := signedHeaders + ParamK[i] + ';';
    canonicalURL := canonicalURL + ParamK[i] + ':' +ParamV[i] + #10;
  end;
  signedHeaders := Copy(signedHeaders, 1, Length(signedHeaders) - 1);
  canonicalURL := canonicalURL + #10 + signedHeaders + #10;
  canonicalURL := canonicalURL + CalcSHA256(payload);

  stringToSign := HMACAlgorithm + #10;
  stringToSign := stringToSign + sDate + #10;
  stringToSign := stringToSign + cDate + '/' + RegionName + '/' + ServiceName + '/' + aws4Request + #10;
  stringToSign := stringToSign + CalcSHA256(canonicalURL);

  kSecret := TEncoding.UTF8.GetBytes('AWS4' + SK);
  kDate := CalcHMACSHA256(cDate, kSecret);
  kRegion := CalcHMACSHA256(regionName, kDate);
  kService := CalcHMACSHA256(serviceName, kRegion);
  kSigning := CalcHMACSHA256(aws4Request, kService);
  ksignature := CalcHMACSHA256(stringToSign, kSigning);
  strHexSignature := BytesToHex(ksignature);

  //CURLの場合
  {GoCURL := '/c curl -o "' + SFile + '" "https://' + host +path +'" ';
  GoCURL := GoCURL + '-H "' + ParamK[1] + ': ' + ParamV[1] + '" ';
  GoCURL := GoCURL + '-H "' + ParamK[2] + ': ' + ParamV[2] + '" ';
  GoCURL := GoCURL + '-H "' + ParamK[3] + ': ' + ParamV[3] + '" ';
  GoCURL := GoCURL + '-H "' + ParamK[4] + ': ' + ParamV[4] + '" ';
  GoCURL := GoCURL + '-H "' + ParamK[5] + ': ' + ParamV[5] + '" ';
  GoCURL := GoCURL + '-H "Authorization: ' + HMACAlgorithm + ' Credential=' + AK + '/' + cDate + '/' + regionName + '/' + serviceName + '/' + aws4Request + ' ' + 'SignedHeaders=' + signedHeaders + ' ';
  GoCURL := GoCURL + 'Signature=' + strHexSignature + '" ';
  GoCURL := GoCURL + '-d "';
  GoCURL := GoCURL + AnsiReplaceStr(payload, '"', '¥"') + '"';

  LhInstance := ShellExecute(Application.Handle,
               'open',
               PChar('cmd.exe'),
               PChar(GoCURL),
               nil,
               SW_HIDE);

  if LhInstance <= 32 then result := False else result := True;}


  //リクエストヘッダ生成
  for i := 1 to 5 do
  begin
    Header := TNetHeader.Create(ParamK[i], ParamV[i]);
    Headers := Headers + [Header];
  end;
  AuthString := HMACAlgorithm + ' Credential=' + AK + '/' + cDate + '/' + regionName + '/' +
    serviceName + '/' + aws4Request + ',' + 'SignedHeaders=' + signedHeaders + ',' + 'Signature=' + strHexSignature;
  Header.Create('Authorization', AuthString);
  Headers := Headers + [Header];

  Result := False;
  HttpClient := THttpClient.Create;
  Stream := TStringStream.Create(payload, TEncoding.UTF8);
  try
    try
      //リクエスト送信
      Response := HttpClient.Post('https://' + host + path, Stream, nil, Headers);

      //結果保存
      TFile.WriteAllText(SFile, Response.ContentAsString(), TEncoding.UTF8);

      if Response.GetStatusCode = 200 then
        Result := True
    except
      on E: Exception do
      begin
        //とりあえずエラー内容をファイルに書き出しておく
        TFile.WriteAllText(SFile, E.Message, TEncoding.UTF8);
        TFile.AppendAllText(SFile, E.StackTrace, TEncoding.UTF8);
      end;
    end;
  finally
    Stream.Free;
    HttpClient.Free;
  end;

end;

//バイト列を16進文字列にする
function BytesToHex(const bytes: TBytes): string;
var
  i: Integer;
begin
  Result := '';
  for i := 0 to Length(bytes) - 1 do
  begin
    Result := Result + IntToHex(bytes[i], 2);
  end;
  Result := LowerCase(Result);
end;

//SHA256ハッシュを計算する
function CalcSHA256(const data: string): string;
var
  Hash: THashSHA2;
begin
  Hash := THashSHA2.Create(SHA256);
  Hash.Update(data);
  Result := Hash.HashAsString;
end;

//SHA256ハッシュを利用してHMACを計算する
function CalcHMACSHA256(const data: string; key: TBytes): TBytes;
var
  Hash: THashSHA2;
begin
  Hash := THashSHA2.Create(SHA256);
  Result := Hash.GetHMACAsBytes(data, key)
end;


end.
